@@ -13,7 +13,8 @@ from xraydb import (chemparse, material_mu, material_mu_components,
                     atomic_symbol, atomic_mass, atomic_density, xray_edges,
                     xray_edge, xray_lines, xray_line, fluor_yield,
                     ck_probability, core_width, guess_edge,
-                    xray_delta_beta, XrayDB)
+                    xray_delta_beta, mirror_reflectivity,
+                    ionchamber_fluxes, XrayDB)
 
 
 from xraydb.xray import chantler_data
@@ -24,7 +25,8 @@ def test_atomic_data():
     assert atomic_mass(45) > 102.8
     assert atomic_mass(45) < 103.0
     assert atomic_density(29) == atomic_density('cu')
-    assert atomic_density(22) > 4.51
+    assert atomic_density(22) > 4.50
+    assert atomic_density(79) > 19.2
 
 
 def test_edge_energies():
@@ -504,6 +506,24 @@ def test_delta_beta():
                      1.84449823e-06, 1.82722124e-06, 1.81015203e-06,
                      1.79327486e-06, 1.77659378e-06, 1.76011187e-06])
 
+    beta_photo = np.array([3.90526978e-07, 3.86063935e-07, 3.81665372e-07,
+                        3.77336646e-07, 3.73071089e-07, 3.68866519e-07,
+                        3.64729098e-07, 3.60655470e-07, 3.56639426e-07,
+                        3.52684594e-07, 3.48796990e-07, 3.44963677e-07,
+                        3.41183740e-07, 3.37480438e-07, 3.33829590e-07,
+                        3.30228863e-07, 3.26702653e-07, 3.23242941e-07,
+                        3.19829864e-07, 3.16477854e-07, 3.13244987e-07,
+                        3.10054443e-07, 3.06905547e-07, 3.03998036e-07,
+                        3.01199959e-07, 2.98435746e-07, 2.96106878e-07,
+                        2.94958918e-07, 2.93820876e-07, 2.93903973e-07,
+                        3.11478882e-07, 2.10154368e-06, 2.11105335e-06,
+                        2.09391918e-06, 2.07224074e-06, 2.05084613e-06,
+                        2.02973085e-06, 2.00827838e-06, 1.98675441e-06,
+                        1.96551940e-06, 1.94456638e-06, 1.92374766e-06,
+                        1.90320773e-06, 1.88294210e-06, 1.86292819e-06,
+                        1.84314908e-06, 1.82363251e-06, 1.80437431e-06,
+                        1.78600492e-06, 1.76816730e-06, 1.75055477e-06])
+
     atten = np.array([0.00365468, 0.00368604, 0.00371757, 0.00374933,
                       0.00378128, 0.0038134 , 0.0038457 , 0.00387818,
                       0.00391084, 0.00394364, 0.00397654, 0.00400961,
@@ -522,5 +542,38 @@ def test_delta_beta():
     d, b, a = xray_delta_beta('Fe2O3', 5.25, en)
 
     assert_allclose(delta, d, rtol=0.005)
-    assert_allclose(beta, b, rtol=0.005)
+    assert_allclose(beta_photo, b, rtol=0.005)
     assert_allclose(atten, a, rtol=0.005)
+
+def test_mirror_reflectivity():
+    rh1 = np.array([0.97199571, 0.97748356, 0.95360848, 0.92357963, 0.93588329,
+                    0.94249572, 0.94771043, 0.95137756, 0.95416567, 0.95693941,
+                    0.95906606, 0.9606963 , 0.96193867, 0.96298571, 0.9637671 ,
+                    0.96434822, 0.96458685, 0.96443861, 0.963873  , 0.96273628,
+                    0.9607325 , 0.95705164, 0.94578432, 0.69383757, 0.65937538,
+                    0.57222587, 0.40829232, 0.26919575, 0.18904094, 0.14024917])
+    r1 = mirror_reflectivity('Rh', 0.0025, np.arange(1000, 31000, 1000))
+    assert_allclose(rh1, r1, rtol=0.005)    
+
+    assert_allclose(mirror_reflectivity('Pt', 0.001, 80000), 0.74, rtol=0.005)
+
+    
+def test_ionchamber_fluxes():
+        ic1 = ionchamber_fluxes(gas='helium', volts=1.25, length=200.0,
+                              energy=10000.0, sensitivity=1.e-9)
+
+        assert_allclose(ic1.photo, 16110895.3, rtol=0.01)
+        assert_allclose(ic1.transmitted, 15316549138.8, rtol=0.01)
+
+        ic2 = ionchamber_fluxes(gas='nitrogen', volts=1.25, length=200.0,
+                                energy=10000.0, sensitivity=1.e-9)
+
+        assert_allclose(ic2.photo, 13575282.2, rtol=0.01)
+        assert_allclose(ic2.incident, 23102328.0, rtol=0.01)
+        
+        ic3 = ionchamber_fluxes(gas={'nitrogen':0.5, 'helium': 0.5}, volts=1.25,
+                                length=200.0, energy=10000.0, sensitivity=1.e-9)
+
+        assert_allclose(ic3.photo, 14843088.8, rtol=0.01)
+        assert_allclose(ic3.incident, 7737855176.4, rtol=0.01)
+        assert_allclose(ic3.transmitted, 7662654298.8, rtol=0.01)
