@@ -12,9 +12,10 @@
 
 from .xraydb import XrayDB
 
-from .chemparser import chemparse
-from .materials import (material_mu, material_mu_components,
-                        get_material, add_material)
+from .chemparser import chemparse, validate_formula
+
+from .materials import (material_mu, material_mu_components, get_materials,
+                        get_material, find_material, add_material)
 
 from .xray import (atomic_number, atomic_symbol, atomic_mass,
                    atomic_density, xray_edges, xray_edge, xray_lines,
@@ -22,7 +23,9 @@ from .xray import (atomic_number, atomic_symbol, atomic_mass,
                    f0_ions, chantler_energies, f1_chantler, f2_chantler,
                    mu_chantler, mu_elam, coherent_cross_section_elam,
                    incoherent_cross_section_elam, guess_edge,
-                   xray_delta_beta, get_xraydb)
+                   xray_delta_beta, get_xraydb,
+                   mirror_reflectivity, ionchamber_fluxes,
+                   ionization_potential)
 
 from ._version import get_versions
 __version__ = get_versions()['version']
