@@ -1,10 +1,19 @@
 import sys
+from collections import namedtuple
 import numpy as np
 
-from .utils import (R_ELECTRON_CM, AVOGADRO, PLANCK_HC, index_nearest)
-from .xraydb import  XrayDB,  XrayLine
+from .utils import (R_ELECTRON_CM, AVOGADRO, PLANCK_HC,
+                    QCHARGE, SI_PREFIXES, index_nearest)
+
+
+from .xraydb import XrayDB,  XrayLine
 from .chemparser import chemparse
 
+fluxes = namedtuple('IonChamberFluxes', ('photo',
+                                         'incident',
+                                         'transmitted'))
+
+
 _edge_energies = {'k': np.array([-1.0, 13.6, 24.6, 54.7, 111.5, 188.0,
                                  284.2, 409.9, 543.1, 696.7, 870.2, 1070.8,
                                  1303.0, 1559.0, 1839.0, 2145.5, 2472.0,
@@ -115,7 +124,16 @@ _edge_energies = {'k': np.array([-1.0, 13.6, 24.6, 54.7, 111.5, 188.0,
 _xraydb = None
 
 def get_xraydb():
-    """return xraydb"""
+    """return instance of the XrayDB
+
+    Returns:
+        XrayDB
+
+    Example:
+        >>> import xraydb
+        >>> xdb = xraydb.get_xraydb()
+
+    """
     global _xraydb
     if _xraydb is None:
         _xraydb = XrayDB()
@@ -570,6 +588,46 @@ def core_width(element, edge=None):
     xdb = get_xraydb()
     return xdb.corehole_width(element, edge=edge)
 
+def ionization_potential(gas):
+    """return effective ionization potential for a gas, as appropriate for
+    ionization chambers in the linear regime (not 'proportional counter' regime)
+
+    Args:
+        gas (string):  name of gas
+
+    Returns:
+        ionization potential in eV
+
+    Notes:
+       Data from G. F. Knoll, Radiation Detection and Measurement, Table 5-1, and
+       from ICRU Report 31, 1979.  Supported gas names and effective potentials:
+
+           ==================== ================
+            gas names            potential (eV)
+           -------------------- ----------------
+            hydrogen, H           36.5
+            helium, He            41.3
+            nitrogren, N, N2      34.8
+            oxygen, O, O2         30.8
+            neon, Ne              35.4
+            argon, Ar             26.4
+            krypton, Kr           24.4
+            xenon, Xe             22.1
+            air                   33.8
+            methane, CH4          27.3
+            carbondioxide, CO2    33.0
+           ==================== ================
+
+       If the gas is not recognized the default value of 32.0 eV will be returned.
+
+    """
+    xdb = get_xraydb()
+    try:
+        return xdb.ionization_potential(gas)
+    except:
+        return 32.0
+
+
 def guess_edge(energy, edges=['K', 'L3', 'L2', 'L1', 'M5']):
     """guess an element and edge based on energy (in eV)
 
@@ -635,13 +693,12 @@ class Scatterer:
         self.symbol = symbol
         self.number = atomic_number(symbol)
         self.mass   = atomic_mass(symbol)
-        self.f1     = chantler_data(symbol, energy, 'f1')
-        self.f1     = self.f1 + self.number
-        self.f2     = chantler_data(symbol, energy, 'f2')
+        self.f1     = self.number + f1_chantler(symbol, energy)
+        self.f2     = f2_chantler(symbol, energy)
         self.mu_photo = chantler_data(symbol, energy, 'mu_photo')
         self.mu_total = chantler_data(symbol, energy, 'mu_total')
 
-def xray_delta_beta(material, density, energy, photo_only=False):
+def xray_delta_beta(material, density, energy):
     """anomalous components of the index of refraction for a material,
     using the tabulated scattering components from Chantler.
 
@@ -649,8 +706,7 @@ def xray_delta_beta(material, density, energy, photo_only=False):
        material:   chemical formula  ('Fe2O3', 'CaMg(CO3)2', 'La1.9Sr0.1CuO4')
        density:    material density in g/cm^3
        energy:     x-ray energy in eV
-       photo_only: boolean for returning photo cross-section component only
-                   if False (default), the total cross-section is returned
+
     Returns:
       (delta, beta, atlen)
 
@@ -666,8 +722,10 @@ def xray_delta_beta(material, density, energy, photo_only=False):
     Adapted from code by Yong Choi
 
     """
+
     lamb_cm = 1.e-8 * PLANCK_HC / energy # lambda in cm
     elements = []
+
     for symbol, number in chemparse(material).items():
         elements.append((number, Scatterer(symbol, energy)))
 
@@ -680,8 +738,170 @@ def xray_delta_beta(material, density, energy, photo_only=False):
         total_mass += number * scat.mass
 
     scale = lamb_cm * lamb_cm * R_ELECTRON_CM / (2*np.pi*total_mass)
-    delta = delta * scale
-    beta  = beta_total * scale
-    if photo_only:
-        beta  = beta_photo * scale
-    return delta, beta, lamb_cm/(4*np.pi*beta)
+    delta  *= scale
+    beta_photo *= scale
+    beta_total *= scale
+    if isinstance(beta_total, np.ndarray):
+        beta_total[np.where(beta_total<1.e-99)] = 1.e-99
+    else:
+        beta_total = max(beta_total, 1.e-19)
+    return delta, beta_photo, lamb_cm/(4*np.pi*beta_total)
+
+def mirror_reflectivity(formula, theta, energy, density=None,
+                        roughness=0.0, polarization='s'):
+    """mirror reflectivity for a thick, singl-layer mirror.
+
+    Args:
+       formula (string):           material name or formula ('Si', 'Rh', 'silicon')
+       theta (float or nd-array):  mirror angle in radians
+       energy (float or nd-array): X-ray energy in eV
+       density (float or None):    material density in g/cm^3
+       roughness (float):          mirror roughness in Angstroms
+       polarization ('s' or 'p'):  mirror orientation relative to X-ray polarization
+
+    Returns:
+       mirror reflectivity values
+
+    Notes:
+       1. only one of theta or energy can be an nd-array
+       2. density can be `None` for known materials
+       3. polarization of 's' puts the X-ray polarization along the mirror
+          surface, 'p' puts it normal to the mirror surface. For
+          horizontally polarized X-ray beams from storage rings, 's' will
+          usually mean 'vertically deflecting' and 'p' will usually mean
+          'horizontally deflecting'.
+    """
+    from .materials import get_material
+    if density is None:
+        formula, density = get_material(formula)
+
+    delta, beta, _ = xray_delta_beta(formula, density, energy)
+    n = 1 - delta - 1j*beta
+
+    # kiz is k in air/vacuum,  with n = 1.
+    # ktz is k in mirror material, with n < 1.
+    # Note: kiz and ktz should have a factor of (2*pi/lambda)
+    # but this will cancel in the calculation of r_amp, so we drop it here.
+    kiz = np.sin(theta)
+    ktz = np.sqrt(n**2 - np.cos(theta)**2)
+
+    # polarization correction will be tiny for small angles
+    if polarization == 'p':
+        ktz = ktz / n
+
+    r_amp = (kiz - ktz)/(kiz + ktz)
+    if roughness > 1.e-12:
+        r_amp = r_amp * np.exp(-2*(roughness**2*kiz*ktz))
+    return (r_amp*r_amp.conjugate()).real
+
+
+def ionchamber_fluxes(gas='nitrogen', volts=1.0, length=100.0,
+                      energy=10000.0, sensitivity=1.e-6,
+                      sensitivity_units='A/V'):
+    
+    """return ion chamber fluxes for a gas or mixture of gases, ion chamber
+    length, X-ray energy, recorded voltage and current amplifier sensitivity.
+
+    Args:
+        gas (string or dict):  name or formula of fill gas (see note 1) ['nitrogen']
+        volts (float):  measured voltage output of current amplifier  [1.0]
+        length (float): active length of ion chamber in mm [100]
+        energy (float): X-ray energy in eV [10000]
+        sensitivity (float): current amplifier sensitivity [1.e-6]
+        sensitivity_units (string): units of current amplifier sensitivity
+                                    (see note 2 for options) ['A/V']
+
+    Returns:
+        named tuple IonchamberFluxes with fields
+
+            `photo`       flux absorbed by photo-electric effect in Hz,
+            
+            `incident`    flux of beam incident on ion chamber in Hz,
+            
+            `transmitted` flux of beam output of ion chamber in Hz 
+
+    Notes:
+       1. The gas argument can either be a string for the name of chemical
+          formula for the gas, or dictionary with keys that are gas names or
+          formulas and values that are the relative fraction for mixes gases.
+
+          The gas formula is used in two ways:
+             a) to get the photo- and total- absorption coefficient, and
+             b) to get the effective ionization potential for the gas.
+
+          The effective ionization potentials are known for a handful of
+          gases (see `ionization_potential` function), all ranging between
+          20 and 45 eV. For unknown gases the value of 32.0 eV will be
+          used.
+
+       2. The `sensitivity` and `sensitivity_units` arguments have some overlap
+          to specify the sensitivity or gain of the current amplifier.
+          Generally, the units in `A/V`, but you can add a common SI prefix of
+ 
+          'p', 'pico', 'n', 'nano', '\u03bc', 'u', 'micro', 'm', 'milli'
+          
+          so that, for example
+             ionchamber_fluxes(..., sensitivity=1.e-6)
+          and 
+             ionchamber_fluxes(..., sensitivity=1, sensitivity_units='uA/V')
+
+          will both give a sensitivity of 1 microAmp / Volt . 
+               
+    Examples:
+        >>> ionchamber_fluxes(gas='helium', volts=1.25, length=200.0,
+                              energy=10000.0, sensitivity=1.e-9)
+        IonChamberFluxes(photo=16110895.3, incident=15452608024.6, transmitted=15316549138.8)
+
+        >>> ionchamber_fluxes(gas='nitrogen', volts=1.25, length=200.0,
+                              energy=10000.0, sensitivity=1.e-9)
+        IonChamberFluxes(photo=13575282.2, incident=23102328.0, transmitted=8759458.7)
+
+        >>> ionchamber_fluxes(gas={'nitrogen':0.5, 'helium': 0.5}, volts=1.25,
+                              length=200.0, energy=10000.0, sensitivity=1.e-9)
+        IonChamberFluxes(photo=14843088.8, incident=7737855176.4, transmitted=7662654298.8)
+
+    
+    """
+    from .materials import material_mu
+
+    fin = fout = fphoto = 0.0
+
+    units = sensitivity_units.replace('Volts', 'V').replace('Volt', 'V')
+    units = units.replace('Amperes', 'A').replace('Ampere', 'A')
+    units = units.replace('Amps', 'A').replace('Amp', 'A')    
+    units = units.replace('A/V', '')
+    sensitivity *= SI_PREFIXES.get(units, 1)
+
+    if isinstance(gas, str):
+        gas = {gas: 1.0}
+    
+    gas_total = 0.0
+    gas_comps = []
+    for gname, frac in gas.items():
+        ionpot = ionization_potential(gname)
+        if gname == 'N2': gname = 'nitrogen'
+        if gname == 'O2': gname = 'oxygen'
+        gas_total += frac
+        gas_comps.append((gname, frac, ionpot))
+
+
+    # note on Photo v Total attenuation:
+    # the current is from the photo-electric cross-section, so that
+    #   flux_photo = flux_in * [1 - exp(-t*mu_photo)]
+    # while total attenuation means
+    #   flux_out = flux_in * exp(-t*mu_total)
+
+    for gas, frac, ionpot in gas_comps:
+        mu_photo = material_mu(gas, energy=energy, kind='photo')
+        mu_total = material_mu(gas, energy=energy, kind='total')
+
+        flux_photo = volts * sensitivity * ionpot / (2 * QCHARGE * energy)
+        flux_photo *= (frac/gas_total)
+        flux_in    = flux_photo / (1.0 - np.exp(-length*mu_photo))
+        flux_out   = flux_in * np.exp(-length*mu_total)
+
+        fphoto += flux_photo
+        fin    += flux_in 
+        fout   += flux_out
+
+    return fluxes(photo=fphoto, incident=fin,transmitted=fout)
